package net.sf.itcb.common.server.interceptor;

import net.sf.itcb.common.business.core.ItcbConstants;

import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.oxm.jaxb.Jaxb2Marshaller;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.server.EndpointInterceptor;
import org.springframework.ws.soap.SoapHeader;
import org.springframework.ws.soap.SoapMessage;
import org.springframework.xml.transform.StringSource;

/**
 * Generic class to retrieve the user in soap headers
 * @author Pierre Le Roux
 *
 * @param <T> the header class to use. This class must be a JAXB element known by the headerMarshaller
 */
public abstract class UserHeaderEndpointInterceptor<T> implements EndpointInterceptor {

	private Jaxb2Marshaller headerMarshaller;
	
	@Required
	public void setHeaderMarshaller(Jaxb2Marshaller headerMarshaller) {
		this.headerMarshaller = headerMarshaller;
	}

	@Override
	public boolean handleFault(MessageContext messageContext, Object endpoint)
			throws Exception {
		return true;
	}

	@Override
	public boolean handleRequest(MessageContext messageContext, Object endpoint)
			throws Exception {
		SoapHeader soapHeader = ((SoapMessage) messageContext.getRequest()).getSoapHeader();
		SecurityContext initialSecurityContext = SecurityContextHolder.getContext();
		if( soapHeader != null) {
			String userContextString = soapHeader.getAttributeValue(ItcbConstants.SOAP_USER_CONTEXT);
			
			if(userContextString != null) {
				T userContext = getUserContext(userContextString);
				if(userContext != null) {
					initialSecurityContext.setAuthentication(getAuthentication(userContext));
					MDC.put(ItcbConstants.MDC_USER_KEY, SecurityContextHolder.getContext().getAuthentication().getName());
					return true;
				}
			}
		}
		// Si le user est un user guest alors on charge avec une authentification null et le user à vide
		initialSecurityContext.setAuthentication(null);
		MDC.put(ItcbConstants.MDC_USER_KEY, "");
		return true;
	}

	/**
	 * Retrieve the IT specific user context object from soap header.
	 * @param userContextString
	 * @return the IT specific user context 
	 */
	@SuppressWarnings("unchecked")
	protected T getUserContext(String userContextString) {
		return (T)headerMarshaller.unmarshal(new StringSource(userContextString));
	}

	@Override
	public boolean handleResponse(MessageContext messageContext, Object endpoint)
			throws Exception {
		return true;
	}
	
	/**
	 * This method has to be implemented by subclasses in order to retrieve a formal Authentification (or subclass) object
	 * from an IT specific user context object retrieved in SOAP header.<br/>
	 * 
	 * <b>The following code is an example (with a usercontext class having getters on user, password and a list of roles)</b>
	 * 
	 * <pre>{@code
	 * List<GrantedAuthority> listRole = new ArrayList<GrantedAuthority>();
	 *   for (String role : userContext.getRoles()) {
	 *     listRole.add(new GrantedAuthorityImpl(role));
     *   }
     * UsernamePasswordAuthenticationToken usernamePasswordAuthenticationToken = new UsernamePasswordAuthenticationToken(userContext.getUser(),userContext.getPassword(), listRole);
     * usernamePasswordAuthenticationToken.setDetails(userContext);
     * return usernamePasswordAuthenticationToken;
     * </pre>
	 * @param userContext the specific bean
	 * @return the authentication bean
	 */
	protected abstract Authentication getAuthentication(T userContext);

}
